package hottisScd30

import (
  "log"
  "fmt"
  "bytes"
  "encoding/base64"
  "encoding/binary"
  "udi/database"
)

type hottisScd30Values struct {
  Status uint8
  CO2Conc int32
  Temp int32
  Hum int32
}


func Parse(fPort int, _ []byte, frmPayload string, variables *map[string]database.VariableType, _ *map[string]interface{}, _ *database.Device) error {
  if fPort != 2 {
    return fmt.Errorf("Unexpected fPort %d", fPort)
  }

  b, err := base64.StdEncoding.DecodeString(frmPayload)
  if err != nil {
    return fmt.Errorf("Unable to base64-decode payload: %v", err)
  }

  var values hottisScd30Values
  err = binary.Read(bytes.NewReader(b), binary.LittleEndian, &values)
  if err != nil {
    return fmt.Errorf("Unable to cast into struct: %v", err)
  }
  log.Printf("CO2: %d, Temp: %d, Hum: %d, Status: %d", values.CO2Conc, values.Temp, values.Hum, values.Status)

  return nil
}
